/*
 * driver.c
 *
 *  Created on: Apr 16, 2016
 *      Author: lidq
 */

#include <wiringPi.h>
#include <driver.h>

drvier_car car;

void driver_setup()
{
	wiringPiSetup();

	pinMode(PORT_LEFT_IN1, OUTPUT);
	pinMode(PORT_LEFT_IN2, OUTPUT);
	pinMode(PORT_LEFT_ENV, OUTPUT);

	pinMode(PORT_RIGHT_IN1, OUTPUT);
	pinMode(PORT_RIGHT_IN2, OUTPUT);
	pinMode(PORT_RIGHT_ENV, OUTPUT);

	pinMode(PORT_CS_TRIG, OUTPUT);
	pinMode(PORT_CS_ECHO, INPUT);

	car.speed_left = 0;
	car.speed_right = 0;
	car.timer_env_left = 0;
	car.timer_env_right = 0;

	car.distance = MAX_DISTANCE;
}

void driver_run_env_left()
{
	while (true)
	{
		double delay_h = car.timer_env_left;
		double delay_l = MAX_SPEED - car.timer_env_left;

		int d_h = (int) (delay_h * ENV_TIMER * 100);
		int d_l = (int) (delay_l * ENV_TIMER * 100);

		if (d_h != 0)
		{
			driver_left_env(HIGH);
			usleep(delay_h);
		}

		if (d_l != 0)
		{
			driver_left_env(LOW);
			usleep(delay_l);
		}
	}
}

void driver_run_env_right()
{
	while (true)
	{
		double delay_h = car.timer_env_right;
		double delay_l = MAX_SPEED - car.timer_env_right;

		int d_h = (int) (delay_h * ENV_TIMER * 100);
		int d_l = (int) (delay_l * ENV_TIMER * 100);

		if (d_h != 0)
		{
			driver_right_env(HIGH);
			usleep(delay_h);
		}

		if (d_l != 0)
		{
			driver_right_env(LOW);
			usleep(delay_l);
		}
	}
}

void driver_run_distance()
{
	while (true)
	{

		digitalWrite(PORT_CS_TRIG, HIGH);
		usleep(15);
		digitalWrite(PORT_CS_TRIG, LOW);

		struct timeval start;
		struct timeval end;
		gettimeofday(&start, NULL);
		s64 time_use = 0;
		int ret = 0;
		do
		{
			gettimeofday(&end, NULL);
			time_use = (end.tv_sec - start.tv_sec) * 1000000 + (end.tv_usec - start.tv_usec);

			ret = digitalRead(PORT_CS_ECHO);
			//more than 0.1second
			if (time_use > 100 * 1000)
			{
				car.distance = MAX_DISTANCE;
				break;
			}
		}
		while (ret == 0);

		if (ret == HIGH)
		{
			struct timeval dis_start;
			struct timeval dis_end;
			s64 dis_time = 0;
			gettimeofday(&dis_start, NULL);
			do
			{
				ret = digitalRead(PORT_CS_ECHO);

				gettimeofday(&dis_end, NULL);
				dis_time = (dis_end.tv_sec - dis_start.tv_sec) * 1000000 + (dis_end.tv_usec - dis_start.tv_usec);
				if (dis_time > 100 * 1000)
				{
					car.distance = MAX_DISTANCE;
					break;
				}
			}
			while (ret == HIGH);
			double t = (double) dis_time / 1000000.0;
			car.distance = t * SS_SPEED / 2.0;
		}
		//printf("DIS:\t%f\n", car.distance);
		usleep(DIS_TIMER * 1000);
	}
}

void driver_left_env(int ena)
{
	digitalWrite(PORT_LEFT_ENV, ena);
	//printf("driver_left_env\n");
}

void driver_right_env(int ena)
{
	digitalWrite(PORT_RIGHT_ENV, ena);
	//printf("driver_right_env\n");
}

void driver_left_stop()
{
	digitalWrite(PORT_LEFT_IN1, LOW);
	digitalWrite(PORT_LEFT_IN2, LOW);
	//printf("driver_left_stop\n");
}

void driver_left_forward()
{
	digitalWrite(PORT_LEFT_IN1, HIGH);
	digitalWrite(PORT_LEFT_IN2, LOW);
	//printf("driver_left_forward\n");
}

void driver_left_back()
{
	digitalWrite(PORT_LEFT_IN1, LOW);
	digitalWrite(PORT_LEFT_IN2, HIGH);
	//printf("driver_left_back\n");
}

void driver_right_stop()
{
	digitalWrite(PORT_RIGHT_IN1, LOW);
	digitalWrite(PORT_RIGHT_IN2, LOW);
	//printf("driver_right_stop\n");
}

void driver_right_forward()
{
	digitalWrite(PORT_RIGHT_IN1, HIGH);
	digitalWrite(PORT_RIGHT_IN2, LOW);
	//printf("driver_right_forward\n");
}

void driver_right_back()
{
	digitalWrite(PORT_RIGHT_IN1, LOW);
	digitalWrite(PORT_RIGHT_IN2, HIGH);
}

void driver_set_speed(int s_left, int s_right)
{
	//printf("%d\t%d\n", s_left, s_right);

	car.speed_left = s_left;
	car.speed_right = s_right;

	car.timer_env_left = driver_abs(s_left);
	car.timer_env_right = driver_abs(s_right);

	if (car.speed_left > 0)
	{
		driver_left_forward();
	}
	else if (car.speed_left < 0)
	{
		driver_left_back();
	}
	else if (car.speed_left == 0)
	{
		driver_left_stop();
	}

	if (car.speed_right > 0)
	{
		driver_right_forward();
	}
	else if (car.speed_right < 0)
	{
		driver_right_back();
	}
	else if (car.speed_right == 0)
	{
		driver_right_stop();
	}
}

double driver_impact_checking()
{
	return car.distance;
}

int driver_abs(int val)
{
	if (val < 0)
	{
		return -val;
	}
	return val;
}
