/*
 * e.c
 *
 *  Created on: Apr 12, 2016
 *      Author: lidq
 */

#include <engine.h>

s_engine engine;

void engine_reset(int mode)
{
	engine.mode = mode;
	//

	engine.angle = 0;
	engine.va = MAX_SPEED;

	engine.left_v = MAX_SPEED;
	engine.left_a = 0;
	engine.right_v = MAX_SPEED;
	engine.right_a = 0;

	engine.redirect_leftright = 0;
	engine.redirect_type = 0;
	engine.redirect_reset = 0;

	//

	engine.ctl_direct = 0;
	engine.ctl_radius = 0;
	engine.ctl_rotate_type = 0;
}

void engine_control_set(int mode, int ctl_direct, int ctl_rotate_type, double ctl_radius)
{
	if (engine.mode != mode)
	{
		engine_reset(mode);
	}

	engine.ctl_direct = ctl_direct;
	engine.ctl_rotate_type = ctl_rotate_type;
	engine.ctl_radius = ctl_radius;

	//printf("%d %d %d %f\n", engine.mode, engine.ctl_direct, engine.ctl_rotate_type, engine.ctl_radius);
}

void engine_run()
{
	int i = 0;

	while (1)
	{
		//control mode
		if (engine.mode == 0)
		{
			engine_control(&engine);
		}
		//auto mode
		else if (engine.mode == 1)
		{
			srand((u32) time(0));
			double dis = driver_impact_checking();
			engine_redirection(&engine, dis);
			if (engine.redirect_reset == 0 && i % 20 == 0)
			{
				engine_rand_params(&engine);
			}
			engine_move(&engine);
			i++;
		}

		driver_set_speed((int) engine.left_v, (int) engine.right_v);

		usleep(ENG_TIMER * 1000);
	}
}

void engine_move(s_engine *e)
{
	//front
	if (e->angle != 0 && cos(e->angle) > 0)
	{
		//font left
		if (sin(e->angle) < 0)
		{
			e->left_v = e->va * (1 + sin(e->angle));
			e->right_v = e->va;
		}
		//font right
		else
		{
			e->left_v = e->va;
			e->right_v = e->va * (1 - sin(e->angle));
		}
	}

	double timer = (double) ENG_TIMER / 1000;

	e->left_v = e->left_v + e->left_a * timer;
	e->right_v = e->right_v + e->right_a * timer;

	engine_rechk_speed(e);
}

void engine_rechk_speed(s_engine *e)
{
	if (e->left_v > MAX_SPEED)
	{
		e->left_v = MAX_SPEED;
	}
	if (e->right_v > MAX_SPEED)
	{
		e->right_v = MAX_SPEED;
	}
	if (e->left_v < -MAX_SPEED)
	{
		e->left_v = -MAX_SPEED;
	}
	if (e->right_v < -MAX_SPEED)
	{
		e->right_v = -MAX_SPEED;
	}
}

void engine_rand_params(s_engine *e)
{
	e->angle = 90 - (rand() % 180);
	e->angle = e->angle / 180.0 * Pi;

	e->left_a = MAX_SPEED - (rand() % (MAX_SPEED * 2));
	e->right_a = e->left_a;

}

void engine_redirection(s_engine *e, double dis)
{
	if (dis < MIN_REDIRECT_BIT_DIS)
	{
		//big reset
		if (dis > MIN_REDIRECT_SMALL_DIS && (e->redirect_type == 0 || e->redirect_type == 1))
		{
			e->redirect_type = 1;
			if (e->redirect_leftright)
			{
				e->angle = -90.0 / 180.0 * Pi;
			}
			else
			{
				e->angle = 90.0 / 180.0 * Pi;
			}
		}
		//small
		else
		{
			e->redirect_type = 2;
			e->angle = 0;
			e->left_a = 0;
			e->right_a = 0;
			e->left_v = 0;
			e->right_v = 0;
			if (e->redirect_leftright)
			{
				e->left_v = -MAX_SPEED;
				e->right_v = +MAX_SPEED;
			}
			else
			{
				e->left_v = MAX_SPEED;
				e->right_v = -MAX_SPEED;
			}

		}
		e->redirect_reset = 1;
	}
	else
	{
		e->redirect_leftright = rand() % 2;
		e->redirect_type = 0;
		if (e->redirect_reset == 1)
		{
			e->left_a = MAX_SPEED;
			e->left_v = 0;

			e->right_a = MAX_SPEED;
			e->right_v = 0;

			e->redirect_reset = 0;
		}
	}
}

void engine_control(s_engine *e)
{
	if (e->ctl_rotate_type == 1)
	{
		e->left_v = -MAX_SPEED * e->ctl_radius;
		e->right_v = MAX_SPEED * e->ctl_radius;
		return;
	}

	if (e->ctl_rotate_type == 2)
	{
		e->left_v = MAX_SPEED * e->ctl_radius;
		e->right_v = -MAX_SPEED * e->ctl_radius;
		return;
	}

	if (e->ctl_radius == 0)
	{
		e->left_v = 0;
		e->right_v = 0;
		return;
	}

	//front
	if (e->ctl_direct == 0)
	{

		e->left_v = e->ctl_radius * MAX_SPEED;
		e->right_v = e->ctl_radius * MAX_SPEED;

	}
	//back
	if (e->ctl_direct == 1)
	{
		e->left_v = -e->ctl_radius * MAX_SPEED;
		e->right_v = -e->ctl_radius * MAX_SPEED;
	}
	//left
	if (e->ctl_direct == 2)
	{
		e->left_v = 0;
		e->right_v = e->ctl_radius * MAX_SPEED;

	}
	//right
	if (e->ctl_direct == 3)
	{
		e->left_v = e->ctl_radius * MAX_SPEED;
		e->right_v = 0;
	}
}
