/*
 * list.c
 *
 *  Created on: Oct 9, 2015
 *      Author: lidq
 */

#include "../include/hash_list.h"

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

//构建一个顺序线性表
boolean init_list(s_hash_list *list, int (*compare_key)(), void (*free_key)(), void (*free_data)())
{
	if (list == null)
	{
		return false;
	}

	if (compare_key == null)
	{
		return false;
	}

	list->compare_key = compare_key;
	list->free_key = free_key;
	list->free_data = free_data;

	list->header = null;

	return true;
}

//销毁表
boolean destroy_list(s_hash_list *list)
{
	if (list == null)
	{
		return false;
	}

	if (list->header == null)
	{
		return true;
	}

	clear_list(list);

	return true;
}

//清空表中所有元素
boolean clear_list(s_hash_list *list)
{
	if (list == null)
	{
		return false;
	}

	//从头节点开始释放
	s_hash_list_node *p = list->header;
	while (p != null)
	{
		if (list->free_key != null && p->key != null)
		{
			list->free_key(p->key);
			p->key = null;
		}
		if (list->free_data != null && p->data != null)
		{
			list->free_data(p->data);
			p->data = null;
		}
		s_hash_list_node *t = p;
		p = p->next;
		free(t);
	}
	list->header = null;

	return true;
}

//插入一个新元素
boolean list_insert(s_hash_list *list, void *key, void *value)
{
	if (list == null)
	{
		return false;
	}

	s_hash_list_node *p = list->header;
	if (p == null)
	{
		//申请新节点内存
		s_hash_list_node *p_new = (s_hash_list_node *) malloc(sizeof(s_hash_list_node));
		p_new->key = key;
		p_new->data = value;
		p_new->next = null;
		list->header = p_new;
		return true;
	}

	//如果遇到key相同的节点，则替换值
	while (p->next != null)
	{
		if (list->compare_key(p->key, key) == 0)
		{
			if (list->free_key != null && p->key != null)
			{
				list->free_key(p->key);
				p->key = null;
			}
			if (list->free_data != null)
			{
				list->free_data(p->data);
			}
			p->data = value;
			return true;
		}
		p = p->next;
	}

	//替换值
	if (list->compare_key(p->key, key) == 0)
	{
		if (list->free_key != null && p->key != null)
		{
			list->free_key(p->key);
			p->key = null;
		}
		if (list->free_data != null)
		{
			list->free_data(p->data);
		}
		p->data = value;
		return true;
	}

	//申请新节点内存
	s_hash_list_node *p_new = (s_hash_list_node *) malloc(sizeof(s_hash_list_node));
	p_new->key = key;
	p_new->data = value;
	p_new->next = null;
	p->next = p_new;

	return false;
}

//删除元素
boolean list_delete(s_hash_list *list, void *key)
{
	if (list == null || list->header == null)
	{
		return false;
	}

	//删除头节点
	s_hash_list_node *p = list->header;
	if (list->compare_key(p->key, key) == 0)
	{
		list->header = p->next;

		if (list->free_key != null && p->key != null)
		{
			list->free_key(p->key);
			p->key = null;
		}
		if (list->free_data != null && p->data != null)
		{
			list->free_data(p->data);
			p->data = null;
		}
		free(p);
		return true;
	}

	//查找并删除节点
	while (p->next != null)
	{
		if (list->compare_key(p->key, key) == 0)
		{
			s_hash_list_node *p_del = p->next;
			p->next = p_del->next;

			if (list->free_key != null && p_del->key != null)
			{
				list->free_key(p_del->key);
				p_del->key = null;
			}
			if (list->free_data != null && p_del->data != null)
			{
				list->free_data(p_del->data);
				p_del->data = null;
			}
			free(p_del);
			return true;
		}
		p = p->next;
	}

	return false;
}

//根据关键字取得数据
void* list_get(s_hash_list *list, void *key)
{
	if (list == null)
	{
		return false;
	}

	//查找key的值
	s_hash_list_node *p = list->header;
	while (p != null)
	{
		if (list->compare_key(p->key, key) == 0)
		{
			return p->data;
		}
		p = p->next;
	}

	return null;
}
