/*
 * server.c
 *
 *  Created on: Mar 11, 2016
 *      Author: lidq
 */

#include <server.h>

s_server server;

s_config config;

pthread_mutex_t mlock;

u64 main_pthread = 0;

s_hash_list list;

pthread_t pthread_master_id;

pthread_t thread_engine = 0;
pthread_t thread_engine_run = 0;

pthread_t thread_driver = 0;
pthread_t thread_driver_run = 0;

pthread_t thread_distance = 0;
pthread_t thread_distance_run = 0;

pthread_t pthread_engine_id = 0;
pthread_t pthread_distance_id = 0;
pthread_t pthread_driver_left_id = 0;
pthread_t pthread_driver_right_id = 0;

extern pthread_t pthread_engine_id;

int compare_addr(void *p, void *p2)
{
	if (p < p2)
	{
		return -1;
	}

	if (p > p2)
	{
		return 1;
	}

	return 0;
}

boolean make_socket_non_blocking(int sfd)
{
	int flags, s;

	flags = fcntl(sfd, F_GETFL, 0);
	if (flags == -1)
	{
		printf("fcntl error\n");
		return false;
	}

	flags |= O_NONBLOCK;
	s = fcntl(sfd, F_SETFL, flags);
	if (s == -1)
	{
		printf("fcntl error\n");
		return false;
	}

	return true;
}

boolean server_start()
{
	printf("[ OK ] Initialize server start.\n");

	config_load(&config);

	printf("[ OK ] Load config file.\n");

	sem_init(&server.sem_distance, 0, 1);
	sem_init(&server.sem_driver_left, 0, 1);
	sem_init(&server.sem_driver_right, 0, 1);
	sem_init(&server.sem_engine, 0, 1);
	//服务器允许客户端接入的最大连接数
	sem_init(&server.sem_server, 0, 1);
	//服务器允许客户端接入的最大连接数
	sem_init(&server.sem_connection, 0, config.max_client_count);
	//允许新客户端接入信号量
	sem_init(&server.sem_new_accept, 0, config.max_accept_count);

	pthread_mutex_init(&mlock, NULL);

	init_list(&list, &compare_addr, null, null);

	printf("[ OK ] Load libraries file.\n");

	//套接字
	server.socket_descriptor = socket(AF_INET, SOCK_STREAM, 0);
	if (server.socket_descriptor <= 0)
	{
		printf("[ ERROR ] Socket_descriptor.\n");
		return false;
	}

#ifdef __NON_BLOCK__
	make_socket_non_blocking(server.socket_descriptor);
#endif

	//服务地址与端口
	memset(&server.servaddr, 0, sizeof(server.servaddr));
	server.servaddr.sin_family = AF_INET;
	server.servaddr.sin_addr.s_addr = htonl(INADDR_ANY);
	server.servaddr.sin_port = htons(config.server_port);

	//绑定套接字
	int bind_status = -1;
	for (int i = 0; bind_status == -1 && i < (int) config.bind_wait_secs; i++)
	{
		bind_status = bind(server.socket_descriptor, (struct sockaddr*) &server.servaddr, sizeof(server.servaddr));
		if (bind_status == 0)
		{
			break;
		}
		printf("[ RUNNING ] Waiting for host port ready... %d/%ds\n", (i + 1), config.bind_wait_secs);
		sleep(1);
	}
	if (bind_status == 0)
	{
		printf("[ OK ] Bind host port.\n");
	}
	else
	{
		printf("[ ERROR ] Bind host port.\n");
		return false;
	}

	//开始监听
	int listen_id = listen(server.socket_descriptor, config.max_client_count);
	if (listen_id == -1)
	{
		printf("[ ERROR ] Listen host connection.\n");
		return false;
	}

	main_pthread = pthread_self();

	signal(SIGSEGV, exception_handler);
	signal(SIGSTKFLT, exception_handler);
	signal(SIGPIPE, exception_handler);

	driver_start();
	//启动引擎
	engine_start();

	printf("[ OK ] Server is ready.\n");
	//循环创建新的空闲连接
	while (true)
	{
		sem_wait(&server.sem_server);

		sem_close(&server.sem_connection);
		sem_init(&server.sem_connection, 0, config.max_client_count);
		sem_close(&server.sem_new_accept);
		sem_init(&server.sem_new_accept, 0, config.max_accept_count);

		int t = pthread_create(&server.thread_engine, null, (void*) &server_engine, null);
		if (t < 0)
		{
			printf("pthread_create call error, erro: [%d][%s]\n", errno, strerror(errno));
			sleep(1);
		}
	}

	printf("Server Stop.\n");

	return true;
}

//停止服务
void server_stop()
{
	driver_set_speed(0, 0);

	destroy_list(&list);

	if (server.socket_descriptor > 0)
	{
		//关闭套接字
		int st = close(server.socket_descriptor);
		printf("[ OK ] Close socket descriptor... %d\n", st);
	}

	exit(0);
}

void engine_start()
{
	pthread_create(&thread_engine, null, (void*) &engine_create, null);
}

void driver_start()
{
	driver_setup();

	pthread_create(&thread_driver, null, (void*) &driver_create_distance, null);
	pthread_create(&thread_driver, null, (void*) &driver_create_left, null);
	pthread_create(&thread_driver, null, (void*) &driver_create_right, null);
}

void driver_create_distance()
{
	pthread_distance_id = pthread_self();

	while (true)
	{
		sem_wait(&server.sem_distance);
		sleep(1);
		int t = pthread_create(&thread_driver, null, (void*) &driver_run_distance, null);
		if (t < 0)
		{
			printf("pthread_create call error, erro: [%d][%s]\n", errno, strerror(errno));
			sem_post(&server.sem_distance);
		}
	}
}

void driver_create_left()
{
	pthread_driver_left_id = pthread_self();

	while (true)
	{
		sem_wait(&server.sem_driver_left);
		sleep(1);
		int t = pthread_create(&thread_driver_run, null, (void*) &driver_run_env_left, null);
		if (t < 0)
		{
			printf("pthread_create call error, erro: [%d][%s]\n", errno, strerror(errno));
			sem_post(&server.sem_driver_left);
		}
	}
}

void driver_create_right()
{
	pthread_driver_right_id = pthread_self();

	while (true)
	{
		sem_wait(&server.sem_driver_right);
		sleep(1);
		int t = pthread_create(&thread_driver_run, null, (void*) &driver_run_env_right, null);
		if (t < 0)
		{
			printf("pthread_create call error, erro: [%d][%s]\n", errno, strerror(errno));
			sem_post(&server.sem_driver_right);
		}
	}
}

void engine_create()
{
	pthread_engine_id = pthread_self();

	engine_reset(0);

	while (true)
	{
		sem_wait(&server.sem_engine);
		sleep(1);
		//创建新的等待连接
		int t = pthread_create(&thread_engine_run, null, (void*) &engine_run, null);
		if (t < 0)
		{
			printf("pthread_create call error, erro: [%d][%s]\n", errno, strerror(errno));
			sem_post(&server.sem_engine);
		}
	}

}

void server_engine()
{
	pthread_master_id = pthread_self();
	//循环创建新的空闲连接
	while (true)
	{
		//最多允许接入新客户端数
		sem_wait(&server.sem_connection);
		sem_wait(&server.sem_new_accept);
		//创建新的等待连接
		int t = pthread_create(&server.thread_conn, null, (void*) &server_create_connection, null);
		if (t < 0)
		{
			printf("pthread_create call error, erro: [%d][%s]\n", errno, strerror(errno));
			sem_post(&server.sem_new_accept);
			sem_wait(&server.sem_connection);
			sleep(1);
		}
	}
}

//创建新的等待连接
boolean server_create_connection()
{
#ifdef __NON_BLOCK__
	fd_set connfd;
	FD_ZERO(&connfd);
	FD_SET(server.socket_descriptor, &connfd);

	struct timeval timeout;
	timeout.tv_sec = 1;
	timeout.tv_usec = 0;

	int sret = select(server.socket_descriptor + 1, &connfd, NULL, NULL, &timeout);
	if (sret < 0)
	{
		sem_post(&server.sem_new_accept);
		server_close_socket(-1);
		return false;
	}

	if (sret == 0)
	{
		sem_post(&server.sem_new_accept);
		server_close_socket(-1);
		return false;
	}

	if (!FD_ISSET(server.socket_descriptor, &connfd))
	{
		sem_post(&server.sem_new_accept);
		server_close_socket(-1);
		return false;
	}
#endif

	//等待接入
	int socket = -1;
	for (int i = 0; socket <= 0 && i < SOCLET_NONBLOCK_COUNT; i++)
	{
		socket = accept(server.socket_descriptor, (struct sockaddr*) NULL, NULL);
		usleep(SOCLET_NONBLOCK_USLEEP);
	}

	//一但有客户端接入，释放new_appept信号量，允许创建一个新的空闲连接
	sem_post(&server.sem_new_accept);
	if (socket == -1)
	{
		server_close_socket(socket);
		return false;
	}
	pthread_mutex_lock(&mlock);
	list_insert(&list, (void *) pthread_self(), (void *) (u64) socket);
	pthread_mutex_unlock(&mlock);

	//控制
	ctl_get_data(socket);

	//关闭套接字
	server_close_socket(socket);
	return true;
}

//关闭套接字
void server_close_socket(int socket)
{
	close(socket);
	pthread_mutex_lock(&mlock);
	list_delete(&list, (void *) pthread_self());
	pthread_mutex_unlock(&mlock);
	pthread_detach(pthread_self());
	sem_post(&server.sem_connection);
	pthread_exit(null);
}

void exception_handler(int sig)
{
	if (pthread_self() == main_pthread)
	{
		server_stop();
	}
	else if (pthread_self() == pthread_engine_id)
	{
		printf("Engine Exception %d\n", sig);
		pthread_detach(pthread_self());
		sem_post(&server.sem_engine);
		pthread_exit(null);
	}
	else if (pthread_self() == pthread_distance_id)
	{
		printf("Engine Exception %d\n", sig);
		pthread_detach(pthread_self());
		sem_post(&server.sem_distance);
		pthread_exit(null);
	}
	else if (pthread_self() == pthread_driver_left_id)
	{
		printf("Engine Exception %d\n", sig);
		pthread_detach(pthread_self());
		sem_post(&server.sem_driver_left);
		pthread_exit(null);
	}
	else if (pthread_self() == pthread_driver_right_id)
	{
		printf("Engine Exception %d\n", sig);
		pthread_detach(pthread_self());
		sem_post(&server.sem_driver_right);
		pthread_exit(null);
	}
	else if (pthread_self() == pthread_master_id)
	{
		printf("Server Exception %d\n", sig);
		pthread_detach(pthread_self());
		sem_post(&server.sem_server);
		pthread_exit(null);
	}
	else
	{
		printf("Conn Exception %d\n", sig);
		u64 val = (u64) list_get(&list, (void *) pthread_self());
		server_close_socket((int) val);
	}
}
